@@ -220,6 +220,12 @@ static int Rpar2jvalue(JNIEnv *env, SEXP par, jvalue *jpar, sig_buffer_t *sig, i
   while (p && TYPEOF(p)==LISTSXP && (e=CAR(p))) {
     /* skip all named arguments */
     if (TAG(p) && TAG(p)!=R_NilValue) { p=CDR(p); continue; };
+
+    if (jvpos >= maxpars) {
+	if (maxpars == maxJavaPars)
+	    Rf_error("Too many arguments in Java call. maxJavaPars is %d, recompile rJava with higher number if needed.", maxJavaPars);
+	break;
+    }
     
     _dbg(rjprintf("Rpar2jvalue: par %d type %d\n",i,TYPEOF(e)));
     if (TYPEOF(e)==STRSXP) {
@@ -370,7 +376,6 @@ static int Rpar2jvalue(JNIEnv *env, SEXP par, jvalue *jpar, sig_buffer_t *sig, i
     }
     i++;
     p=CDR(p);
-    if (jvpos >= maxpars) break;
   }
   fintmpo(tmpo);
   return jvpos;
@@ -841,24 +846,11 @@ static SEXP new_jrectRef(JNIEnv *env, jobject a, const char *sig, SEXP dim ) {
   SET_SLOT(oo, install("jsig"), mkString(sig));
   SET_SLOT(oo, install("dimension"), dim);
   
-  UNPROTECT(1); // oo
+  UNPROTECT(1); /* oo */
   return oo;
 }
 
-/**
- * Creates a rectangular array reference of dimension 1
- *
- * @param length the number of objects in the array
- */
-static SEXP new_jrectRef1(JNIEnv *env, jobject a, const char *sig, int length ) {
-	SEXP dim = allocVector( INTSXP, 1) ;
-	PROTECT(dim); 
-	INTEGER(dim)[0] = length ;
-	UNPROTECT(1); // dim 
-	return new_jrectRef( env, a, sig, dim ) ;
-}
-
-// this does not take care of multi dimensional arrays properly
+/* this does not take care of multi dimensional arrays properly */
 
 /**
  * Creates a one dimensionnal java array
